<?php
/**
 * Plugin Name: Simple Location
 * Plugin URI: https://wordpress.org/plugins/simple-location/
 * Description: Adds Location to WordPress
 * Version: 3.3.8
 * Author: David Shanske
 * Author URI: https://david.shanske.com
 * Text Domain: simple-location
 * Domain Path:  /languages
 */

define( 'SLOC_PUBLIC', 1 );

add_action( 'plugins_loaded', array( 'Simple_Location_Plugin', 'init' ) );

// Activation and Deactivation Hooks
register_activation_hook( __FILE__, array( 'Simple_Location_Plugin', 'activate' ) );
register_deactivation_hook( __FILE__, array( 'Simple_Location_Plugin', 'deactivate' ) );


class Simple_Location_Plugin {
	public static $version = '3.3.7';

	public static function activate() {
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-geo-data.php';
		WP_Geo_Data::rewrite();
		flush_rewrite_rules();
	}

	public static function deactivate() {
		flush_rewrite_rules();
	}

	public static function init() {

		load_plugin_textdomain( 'simple-location', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

		// Load stylesheets.
		add_action( 'wp_enqueue_scripts', array( 'Simple_Location_Plugin', 'style_load' ) );

		// Settings Link
		add_action( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( 'Simple_Location_Plugin', 'settings_link' ) );

		// Add Privacy Policy
		add_action( 'admin_init', array( 'Simple_Location_Plugin', 'privacy_declaration' ) );

		// Register Metadata Functions
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-geo-data.php';

		// Venue Taxonomy
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-venue-taxonomy.php';

		// Map Provider Class
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-geo-provider.php';

		// Weather Provider Class
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-weather-provider.php';
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-weather-provider-openweathermap.php';

		// Weather Widget
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-sloc-weather-widget.php';
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-sloc-lastseen-widget.php';
		add_action( 'widgets_init', array( 'Simple_Location_Plugin', 'widgets_init' ) );

		// Map Providers
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-geo-provider-osm.php';
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-geo-provider-google.php';
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-geo-provider-bing.php';

		// Location Provider
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-location-provider.php';

		// API Endpoint under construction
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-rest-geo.php';
		$geo_api = new REST_Geo();

		// Configuration Functions
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-loc-config.php';

		// Add Location Post Meta
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-loc-metabox.php';

		// Add Location Display Functions
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-loc-view.php';

		// Timezone Functions
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-timezone-result.php';
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-loc-timezone.php';
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-post-timezone.php';

	}

	public static function widgets_init() {
		register_widget( 'Sloc_Weather_Widget' );
		register_widget( 'Sloc_Lastseen_Widget' );
	}

	/** Adds link to Plugin Page for Options Page.
	 *
	 * @param array $links Array of Existing Links.
	 * @return array Modified Links.
	 */
	public static function settings_link( $links ) {
		$settings_link = '<a href="admin.php?page=simloc">' . __( 'Settings', 'simple-location' ) . '</a>';
		$links[]       = $settings_link;
		return $links;
	}

	/**
	 * Loads the Stylesheet for the Plugin.
	 */
	public static function style_load() {
		wp_enqueue_style( 'simple-location', plugin_dir_url( __FILE__ ) . 'css/location.min.css', array(), self::$version );
	}

	public static function privacy_declaration() {
		if ( function_exists( 'wp_add_privacy_policy_content' ) ) {
			$content = __(
				'Location and weather data is optionally stored for all posts, attachments, and comments. Location data is extracted from uploaded images along with other metadata. This
				data can be removed prior to uploading if tyou do not wish this to be stored. There are options to display this information or hide it.', 'simple-location'
			);
			wp_add_privacy_policy_content(
				'Simple Location',
				wp_kses_post( wpautop( $content, false ) )
			);
		}
	}

}


if ( ! function_exists( 'ifset' ) ) {
	function ifset( &$var, $default = false ) {
		return isset( $var ) ? $var : $default;
	}
}
